@@ -1,11 +1,11 @@
 package Mojolicious::Plugin::Piwik;
 use Mojo::Base 'Mojolicious::Plugin';
-use Mojo::Util qw/deprecated/;
+use Mojo::Util qw/deprecated quote/;
 use Mojo::ByteStream 'b';
 use Mojo::UserAgent;
 use Mojo::IOLoop;
 
-our $VERSION = '0.22';
+our $VERSION = '0.23';
 
 # Todo:
 # - Better test tracking API support
@@ -16,7 +16,8 @@ our $VERSION = '0.22';
 #   http://piwik.org/docs/ecommerce-analytics/
 # - Improve error handling.
 # - Introduce piwik_widget helper
-
+# - Support site_id and url both in piwik('track_script')
+#   shortcut and in piwik_tag 'as_script'
 
 # Register plugin
 sub register {
@@ -33,6 +34,9 @@ sub register {
   my $embed = $plugin_param->{embed} //
     ($mojo->mode eq 'production' ? 1 : 0);
 
+  # No script route defined
+  my $script_route = 0;
+
   # Add 'piwik_tag' helper
   $mojo->helper(
     piwik_tag => sub {
@@ -43,21 +47,35 @@ sub register {
       # Controller
       my $c = shift;
 
+      # Per default integrate as inline
+      my $as_script = 0;
+
       # Use opt-out tag
       my %opt;
-      if ($_[0] && index(lc $_[0], 'opt-out') == 0) {
-        my $opt_out = shift;
+      if ($_[0]) {
+        if (index(lc $_[0], 'opt-out') == 0) {
+          my $opt_out = shift;
 
-        # Get iframe content
-        my $cb = ref $_[-1] eq 'CODE' ? pop : 0;
+          # Get iframe content
+          my $cb = ref $_[-1] eq 'CODE' ? pop : 0;
 
-        # Accept parameters
-        %opt = @_;
-        $opt{out} = $opt_out;
-        $opt{cb}  = $cb;
+          # Accept parameters
+          %opt = @_;
+          $opt{out} = $opt_out;
+          $opt{cb}  = $cb;
 
-        # Empty arguments
-        @_ = ();
+          # Empty arguments
+          @_ = ();
+        }
+
+        # Get a CSP compliant script tag
+        elsif (lc($_[0]) eq 'as-script') {
+
+          $as_script = 1;
+
+          # Empty arguments
+          @_ = ();
+        }
       };
 
       my $site_id = shift || $plugin_param->{site_id} || 1;
@@ -66,16 +84,19 @@ sub register {
       # No piwik url
       return b('<!-- No Piwik-URL given -->') unless $url;
 
-      my $prot = 'http';
+      # Clear URL
+      ($url, my $prot) = _clear_url($url);
 
-      # Clear url
-      for ($url) {
-        if (s{^http(s?):/*}{}i) {
-          $prot = 'https' if $1;
+      # Load as script
+      if ($as_script) {
+        unless ($script_route) {
+          $c->app->log->error('No shortcut for track_script defined');
+          return '';
         };
-        s{piwik\.(?:php|js)$}{}i;
-        s{(?<!/)$}{/};
-      };
+
+        return b("<script src=\"$prot://${url}piwik.js\" async defer></script>" .
+                   '<script src="' . $c->url_for('piwik_track_script')->to_abs . '"></script>');
+      }
 
       # Render opt-out tag
       if (my $opt_out = delete $opt{out}) {
@@ -119,6 +140,54 @@ SCRIPTTAG
     });
 
 
+  # Add piwik shortcut
+  $mojo->routes->add_shortcut(
+    piwik => sub {
+      my $r = shift;
+      my $name = shift // 'unknown';
+
+      # Add track script route
+      if ($name eq 'track_script') {
+
+        my $site_id = $plugin_param->{site_id} || 1;
+        my $url     = $plugin_param->{url};
+
+        unless ($url) {
+          $mojo->log->error('No URL defined for Matomo (Piwik) instance');
+          return;
+        };
+
+        # Clear URL
+        ($url, my $prot) = _clear_url($url);
+
+        $script_route = 1;
+
+        # Return track_script page
+        return $r->to(
+          cb => sub {
+            my $c = shift;
+
+            # Cache for three hours
+            $c->res->headers->cache_control('max-age=' . (60 * 60 * 3));
+
+            # Render tracking code
+            return $c->render(
+              format => 'js',
+              text => 'var _paq=window._paq||[];' .
+                "_paq.push(['setTrackerUrl','$prot://${url}piwik.php']);" .
+                "_paq.push(['setSiteId',$site_id]);" .
+                q!_paq.push(['trackPageView']);! .
+                q!_paq.push(['enableLinkTracking']);!
+              );
+          }
+        )->name('piwik_track_script');
+      };
+
+      $mojo->log->error("Unknown Piwik shortcut " . quote($name));
+      return;
+    }
+  );
+
   # Add 'piwik->api' helper
   $mojo->helper(
     'piwik.api' => sub {
@@ -355,6 +424,21 @@ sub _prepare_response {
 };
 
 
+sub _clear_url {
+  my $url = shift;
+  my $prot = 'http';
+
+  # Clear url
+  for ($url) {
+    if (s{^http(s?):/*}{}i) {
+      $prot = 'https' if $1;
+    };
+    s{piwik\.(?:php|js)$}{}i;
+    s{(?<!/)$}{/};
+  };
+  return ($url, $prot);
+};
+
 1;
 
 
@@ -432,11 +516,11 @@ C<url> - URL of your Matomo (Piwik) instance.
 
 =item
 
-C<site_id> - The id of the site to monitor. Defaults to 1.
+C<site_id> - ID of the site to monitor. Defaults to 1.
 
 =item
 
-C<embed> - Activates or deactivates the embedding of the script tag.
+C<embed> - Activates or deactivates the embedding of Piwik tag.
 Defaults to a C<true> value if Mojolicious is in production mode,
 defaults to a C<false> value otherwise.
 
@@ -470,6 +554,20 @@ was registered.
 This tag should be included at the bottom
 of the body tag of each website you want to analyse.
 
+The special C<as-script> tag renders the tracking code for
+the plugin-wide C<site_id> and C<url> as an external resource.
+This is required to embed the tracking code
+in pages with strict L<Content Security Policies|https://matomo.org/faq/general/faq_20904/>.
+To make this work, it is necessary to define the C<track_script> L<shortcut|/SHORTCUTS>
+and to add the domain of your Matomo (Piwik) instance to your
+CSP as a valid C<script-src> and potentially C<img-src>.
+
+  # In your application
+  $app->routes->route('/tracking.js')->piwik('track_script');
+
+  # In Template:
+  %= piwik_tag 'as-script'
+
 The special C<opt-out> tag renders an
 L<iframe|https://matomo.org/faq/general/faq_20000/>
 helping your visitors to disallow tracking via javascript.
@@ -606,6 +704,26 @@ excluding the callback.
 B<This helper is EXPERIMENTAL and may change without warnings!>
 
 
+=head1 SHORTCUTS
+
+=head2 piwik
+
+  $app->routes->route('/tracking.js')->piwik('track_script');
+
+Defines a C<piwik> shortcut for routes, accepting the following
+route names:
+
+=over 2
+
+=item
+
+C<track_script> - Returns a JavaScript file containing the
+tracking code for C<site_id> and C<url>.
+
+=back
+
+B<The shortcut is EXPERIMENTAL and may change without warnings!>
+
 =head1 LIMITATIONS
 
 The plugin currently lacks support for eCommerce tracking.
@@ -650,7 +768,7 @@ Piwik, especially if you are located in the European Union.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2012-2018, L<Nils Diewald|http://nils-diewald.de/>.
+Copyright (C) 2012-2018, L<Nils Diewald|https://nils-diewald.de/>.
 
 This program is free software, you can redistribute it and/or
 modify it under the terms of the Artistic License version 2.0.
